#[non_exhaustive]pub enum Mode {
Read,
Write,
}Expand description
Choose how opened files can be accessed.
Pass a Mode value to GsdFile::open.
In the Mode::Read mode, you can call methods that read the file, such as
GsdFile::find_chunk and GsdFile::iter_scalars. Calling methods that
write the file, such as GsdFile::write_scalars or GsdFile::sync_all will
result in an error.
In the Mode::Write mode, you can call both read and write methods.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.