#[non_exhaustive]pub enum AppendError {
InconsistentLength(String, String),
Write(WriteError),
Encode(EncodeError),
Sync(SyncError),
ChunkTooLarge(usize, String, TryFromIntError),
}Expand description
Errors that can occur while appending a frame to a HOOMD GSD file.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
InconsistentLength(String, String)
This data chunk does not match the dimensions of those previously written.
Write(WriteError)
Cannot write to the file.
Encode(EncodeError)
Cannot encode data to write.
Sync(SyncError)
Cannot synchronize data to the file.
ChunkTooLarge(usize, String, TryFromIntError)
Too many entries to write.