#[non_exhaustive]pub enum Dimensions {
Two,
Three,
}Expand description
The number of dimensions in the GSD file.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
Two
All particle z coordinates and the box z should be zero.
Three
Particle positions and the box shape can have non-zero z values.